@@ -1,10 +1,6 @@
 # Define the __all__ variable
-__all__ = ["csvutil", "genutil", "jsonutil", "pdfutil"]
-
-import time
-import numpy
-import pandas
-import matplotlib
+__all__ = ["csvutil", "genutil", "jsonutil", "pdfutil", "mf4util"]
+__version__ = "2025.8.11.4"
 
 #s = time.time()
 #print('[pvevti] Import csvutil module')
