@@ -2,9 +2,11 @@
 A library of general utilities focusing mainly on pandas, numPy, csvutil, and jsonutil integration.
 For help, explore the readme and the docs markdown files in the package source.
 """
-from pandas import DataFrame, Series
+from pandas import DataFrame, Series, concat, core
 from numpy import square, radians, append, cos, sqrt
 
+debugInfo = False
+
 geoFence = [
     {
         "name": "PKMR",
@@ -149,13 +151,55 @@ def verticalTicks(data):
         output.append(round(data_min+i*data_range/5, 5))
     return output
 
-def extractE9Serial(path_or_filename:str):
+class Date():
+    def __init__(self, month:int=1, day:int=1, year:int=1970):
+        self.month = month
+        self.year =  year
+        self.day =   day
+
+    def __str__(self):
+        return "{}/{}/{}".format(self.month, self.day, self.year)
+    
+    def getDay(self):
+        return self.day
+    
+    def getMonth(self):
+        return self.month
+    
+    def getYear(self):
+        return self.year
+
+    def daysAfterZero(self):
+        monthLengths = [31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31]
+        daysFromMonths = sum(monthLengths[0:(self.month-1)])
+        return int(self.year * 365.25) + daysFromMonths + self.day
+        
+    def daysAfterEpoch(self):
+        monthLengths = [31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31]
+        daysFromMonths = sum(monthLengths[0:(self.month-1)])
+        return int((self.year-1970) * 365.25) + daysFromMonths + self.day
+
+    def extractDate(path_or_filename:str):
+        strs = path_or_filename.split('/')[-1].split('_')
+        lens = [len(item) for item in strs]
+        #print(strs)
+        #print(lens)
+        goal = [4, 2, 2]
+        for idx, item in enumerate(lens):
+            if lens[idx:idx+3] == goal and str.isnumeric(''.join(strs[idx:idx+3])):
+                return Date(int(strs[idx+1]), int(strs[idx+2]), int(strs[idx]))
+        return Date(1, 1, 1970)
+
+def extractE9Serial(path_or_filename:str, standardize:bool=False):
     """
     Given a string input for a path or filename, extracts the E9 serial number and returns it as a pure string.
     """
     if "E9" in path_or_filename:
         idx = path_or_filename.index("E9")
-        return(path_or_filename[idx:(idx+6)])
+        E9str = path_or_filename[idx:(idx+6)]
+        if standardize:
+            E9str = E9str[0:-1]+str(int(int(E9str[-1])/5)*5)
+        return E9str
     elif "704783" in path_or_filename:
         return "VOLVO"
     else:
@@ -198,7 +242,7 @@ def extractVehicleData(E9Serial:str):
         # Volvo
         "VOLVO":  ("VNL860", "Sleeper", 'Dark gray', "Volvo D13TC", "Volvo I-Shift 12-Speed", "2.15")
     }
-    if "E9" in E9Serial:
+    if "E9" in E9Serial and 'xxxx' not in E9Serial:
         E9Serial = E9Serial[0:-1] + str(int(int(E9Serial[-1])/5)*5)
     if E9Serial in data:
         return data[E9Serial]
@@ -461,6 +505,25 @@ def roundCols(df, rounding_accuracy):
     
     return new_df
 
+def parseName(columns, name:str):
+    """
+    Checks if the name string input is a signal in the columns provided.
+    Columns may be of type list or columns object (like df.columns).
+
+    If the name is found, returns the corrected name (including units and capitalization).
+    Else, returns -1.
+
+    The name string can contain the '[units]' portion of a signal name, but does not need to.
+    """
+    if isinstance(columns, core.indexes.base.Index):
+        columns = columns.to_list()
+    columns_new = [str(column).lower().split('[')[0].strip() for column in columns]
+    
+    name = name.lower().split('[')[0]
+    if name.lower().strip() in columns_new:
+        return columns[columns_new.index(name.lower().strip())]
+    return -1
+
 def parseNames(columns, keys):
     """
     Given columns in the form of df.columns, yields the full column names matching the abbreviated key inputs
@@ -580,3 +643,95 @@ def getRoute(df, notFoundStr:str=""):
         if inGeofence(df, route['box']):
             return (route['name'], route['desc'])
     return (notFoundStr, '')
+
+def detectMatching(pathList:list, sort:bool=True):
+    """
+    Yields a sorted list based on run number of only the CSV paths containing the most common serial number in the input list. 
+    """
+    # Reserve serial numbers and counts of each serial number lists
+    serials = []
+    serialCounts = []
+
+    # Loop to find unique serials and count occurrences
+    for path in pathList:
+        serial = extractE9Serial(path, True)
+        if serial != 'E9XXXX':
+            if serial not in serials:
+                serials.append(serial)
+                serialCounts.append(1)
+            else:
+                serialCounts[serials.index(serial)] += 1
+    
+    # Return empty if no results (spare processing)
+    if len(serials) == 0:
+        return []
+    
+    # Data output
+    topSerial = serials[serialCounts.index(max(serialCounts))]
+    if debugInfo == True:
+        print(list(zip(serials, serialCounts)))
+        print('Most frequently occurring: ', topSerial)
+
+    # Create list of only files to sort
+    listToSort = []
+    dates = []
+    runNumbers = []
+
+    # Sanity check dates are the same
+    for path in pathList:
+        if topSerial == extractE9Serial(path, True):
+            listToSort.append(path)
+            runNumbers.append(int(extractGLRunNumber(path)))
+            dates.append(Date.extractDate(path).daysAfterEpoch())
+
+    span = max(dates) - min(dates)
+    if span > 0:
+        print("WARN Selection encompasses {} days.".format(span+1))
+    
+    if sort:
+        sortedList = [path for run, path in sorted(zip(runNumbers, listToSort))]
+        return sortedList
+    else:
+        return listToSort
+    
+def combineDFs(dfs:list=[], runs:list=[], filenames:list=[]):
+    """
+    Combines a list of dataframes `dfs`. 
+    Optionally accepts a list of run numbers `runs`. These may be of type `int` or `str`, but should not include the 'GL' pretext.
+    Returns a new dataframe with the data merged. 
+    """
+    for i in range(0,len(dfs)):
+        # print(len(dfs[i]))
+        if len(runs) == len(dfs):
+            if 'RunNumber[GL]' not in list(dfs[i].columns):
+                dfs[i].insert(1, 'RunNumber[GL]', [str(runs[i])]*len(dfs[i]), allow_duplicates=True)
+            else:
+                dfs[i]['RunNumber[GL]'] = [str(runs[i])]*len(dfs[i])
+        if len(filenames) == len(dfs):
+            if 'FileName[]' not in list(dfs[i].columns):
+                dfs[i].insert(1, 'FileName[]', [str(filenames[i])]*len(dfs[i]), allow_duplicates=True)
+            else:
+                dfs[i]['FileName[]'] = [str(filenames[i])]*len(dfs[i])
+        if 't[s]' in list(dfs[i].columns):
+            if 'RunTime[s]' not in list(dfs[i].columns):
+                dfs[i].insert(0, 'RunTime[s]', dfs[i]['t[s]'].to_list())
+            else:
+                dfs[i]['RunTime[s]'] = dfs[i]['t[s]'].to_list()
+            if i > 0:
+                dfs[i]['t[s]'] += max(dfs[i-1]['t[s]'].to_list()) + 1
+    
+    if len(dfs) > 1:
+        return concat(dfs, ignore_index=True).fillna(0)
+    elif len(dfs) == 1:
+        return dfs[0]
+    else:
+        return DataFrame()
+    
+def subsetDF(df:DataFrame, signals:list):
+    """
+    Returns the dataframe with only the provided signals.
+    Signals can be provided with or without units, and do not require correct capitalization.
+    If a requested signal does not exist in the DF, it is omitted from the result.
+    """
+    signals = [parseName(df.columns, signal) for signal in signals if parseName(df.columns, signal) != -1]
+    return df[signals]
\ No newline at end of file
