@@ -451,6 +451,105 @@ default_powersteering_pdf_config = {
     ]
 }
 
+default_route_conditions_config = {
+    "docTitle": "DefaultRouteConditionsReport",
+    "docSubTitle": "Route Conditions Report",
+    "pages": [
+        # 
+        {
+            "pageName": "Route Overview",
+            "plots": [
+                {
+                    "plotTitle":"Trip Route",
+                    "plotType": "map",
+                    "xData":    "GPS_speed_mph",
+                    "yData":    ["gps_x", "gps_y"]
+                },
+                {
+                    "plotTitle":"Ambient pressure & temperature",
+                    "plotType": "dualline",
+                    "xData":    "t",
+                    "yData":    ["amb_p", "amb_t"],
+                    "filterLength": 40
+                }
+            ]
+        },
+        {
+            "pageName": "Powertrain",
+            "plots": [
+                {
+                    "plotTitle":"Engine Load",
+                    "plotType": "line",
+                    "xData":    "t",
+                    "yData":    ["EngPctLoadAtCurrSpd_Cval"]
+                },
+                {
+                    "plotTitle":"Engine Output",
+                    "plotType": "dualline",
+                    "xData":    "t",
+                    "yData":    ["tbf_trq_full_load_limit", "isp_trq_current"]
+                },
+                {
+                    "plotTitle":"Transmission Mode",
+                    "plotType":"line",
+                    "xData":    "t",
+                    "yData":    "TransCurrentGear"
+                }
+            ]
+        },
+        {
+            "pageName": "Cruise Control",
+            "plots": [
+                {
+                    "plotTitle":"Cruise Control Mode",
+                    "plotType": "line",
+                    "xData":    "t",
+                    "yData":    ["CC_Actv_Stat"]
+                },
+                {
+                    "plotTitle":"Cruise Control Set Speed",
+                    "plotType": "dualline",
+                    "xData":    "t",
+                    "yData":    ["CC_SetSpd_Cval","vehspd_cval_cpc"]
+                },
+                {
+                    "plotTitle":"Cruise Control Activity",
+                    "plotType": "hist",
+                    "xData":    "CC_Actv_Stat"
+                },
+                {
+                    "plotTitle":"Overview",
+                    "plotType": "table",
+                    "yData":    ["CC_actv_stat", "cc_setspd_cval", "vehspd_cval_cpc"]
+                }
+            ]
+        },
+        {
+            "pageName": "Miscellaneous",
+            "plots": [
+                {
+                    "plotTitle":"Aftertreatment",
+                    "plotType": "dualline",
+                    "xData":    "t",
+                    "yData":    ["tmc_status", "dpf_zone_status"]
+                },
+                {
+                    "plotTitle": "Power-Speed Plot",
+                    "plotType" : "scatter",
+                    "xData"    : "EngRPM_Cval_CPC3",
+                    "yData"    : ["HP_CALCULATED"]
+                },
+                {
+                    "plotTitle": "Torque-Speed Plot",
+                    "plotType" : "scatter",
+                    "xData"    : "EngRPM_Cval_CPC3",
+                    "yData"    : ["isp_trq_current"]
+                }
+            ]
+        }
+    ]
+}
+
 default_config = {
     "docTitle": "Default Document Title", "docSubTitle": "Default Document Subtitle",
     "pages": [
@@ -1112,11 +1211,21 @@ class Plot:
 
                 # Plot the data
                 try:
-                    gdf.plot(
+                    p = gdf.plot(
                         ax=ax, markersize=4, column=Series(self.data[0]),
                         cmap='coolwarm', marker='o', alpha=0.8, legend=True, label='GPS Track',
                         legend_kwds={'label': self.columns[0], 'location': 'bottom'}
                     )
+                    max_x, max_y = max(self.data[1]), max(self.data[2])
+                    min_x, min_y = min(self.data[1]), min(self.data[2])
+
+                    span_x = max(max_x - min_x, 0.003)
+                    mid_x = (max_x + min_x)/2
+                    span_y = max(max_y - min_y, 0.003)
+                    mid_y = (max_y + min_y)/2
+                    span = max(span_x, span_y)*1.1
+                    p.set_xlim([mid_x-span/1.8,  mid_x+span/1.8])
+                    p.set_ylim([mid_y-span/2, mid_y+span/2])
                 except Exception as e:
                     print("[{}] Failed to plot data.".format(str(e)))
                 # Attempt to add a Contextily basemap (internet connection reqd.)
@@ -1330,3 +1439,4 @@ def createDocument(df: DataFrame, config: dict, save_path: str=os.path.expanduse
         del cd_page
 
     cd_document.save(save_path)
+
